// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classEventInterface
#define classEventInterface

class EventInterface;

#include <sys/time.h>
#include "EventClock.h"
#include "EventInterfaceList.h"

//! General Event Interface.
/*!
	This is an abstract class used to model an event timeout.

	An class derived from EventInterface must implement the OnEvent() method. This method is called whenever the timeout occurs.
	
	The most useful way to set a timeout is by calling the SetNextEvent() method. It takes as argument the number of milliseconds to wait until calling OnEvent(). Note that the timeout is cleared when OnEvent is called, i.e. if you want OnEvent to be invoked every 100ms, you need to call
		- SetNextEvent(100)

	within the OnEvent() method to set the timeout again.

	Alternatively, one may use the SetNextEventAbsolute() method to set an absolute time for the next event.

	Note that an EventInterface only holds one timeout at the time. Calling SetNextEvent() or SetNextEventAbsolute() several times will only keep the smallest timeout (the event closest to now in time). However, you can create several objects derived from EventInterface and add them to the EventInterfaceList.
*/
class EventInterface {
friend class EventInterfaceList;

private:
	//! Tells whether the event is active.
	bool mEventActive;
	//! The event time.
	EventClock::tTime mEventTime;

protected:
	//! The next EventInterface in the list.
	EventInterfaceList *mEventInterfaceList;
	//! The next EventInterface in the list.
	EventInterface *mEventInterfaceNext;

public:
	//! Constructor.
	EventInterface(EventInterfaceList *evlist);
	//! Destructor.
	~EventInterface();

	//! Returns the EventInterfaceList to which this EventInterface belongs to.
	EventInterfaceList *GetEventInterfaceList() {return mEventInterfaceList;}
	//! Returns the next EventInterface in the list.
	EventInterface *GetEventInterfaceNext() {return mEventInterfaceNext;}

	//! Dispatches the event by calling OnEvent() if necessary.
	void DispatchEvent();
	//! This is called when the event timer expires.
	virtual void OnEvent() = 0;

	//! Sets the next event (usec relative to the current event clock).
	void SetNextEvent(int msec);
	//! Sets the next event (absolute time).
	void SetNextEventAbsolute(EventClock::tTime evtime);

	//! Returns the time of the next event.
	EventClock::tTime GetNextEventTime() const {return mEventTime;}
	//! Returns whether the event is active.
	bool HasNextEvent() const {return mEventActive;}
};

#endif
