// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classIOInterface
#define classIOInterface

class IOInterface;

#include <string>
#include <fstream>
#include <iostream>
#include "IOInterfaceList.h"

//! General IO Interface.
/*!
	An IOInterface object is attached to a file handle. The application should invoke OnInputAvailable whenever the filehandle becomes active. This class then reads the new data and dispatches it.

	Data can be sent using the Send() method.

	All incoming and outgoing data can be logged to a file using SetInputLogFile() and SetOutputLogFile().

	This is an abstract class. You need to overwrite at least the OnDataArrival() and the OnClose() methods.

	\sa IOInterfaceSerial, IOInterfaceList
*/
class IOInterface {
friend class IOInterfaceList;

protected:
	//! The next IOInterface in the list.
	IOInterfaceList *mIOInterfaceList;
	//! The next IOInterface in the list.
	IOInterface *mIOInterfaceNext;

	//! The file handle of the open file/device (or -1 if the file/device is closed).
	int mFileHandle;

	//! The file where incoming traffic is logged.
	std::ofstream mLogInput;
	//! The file where outgoing traffic is logged.
	std::ofstream mLogOutput;

public:
	//! Constructor.
	IOInterface(IOInterfaceList *iolist);
	//! Destructor.
	~IOInterface();

	//! Returns the IOInterfaceList to which this IOInterface belongs to.
	IOInterfaceList *GetIOInterfaceList() {return mIOInterfaceList;}
	//! Returns the next IOInterface in the list.
	IOInterface *GetIOInterfaceNext() {return mIOInterfaceNext;}

	//! (abstract) This is called by the application when the filehandle has become active.
	virtual void OnInputAvailable();
	//! (abstract) This is called by OnInputAvailable() to process data that arrived.
	virtual void OnDataArrival(unsigned char *data, int len) = 0;
	//! (abstract) This is called when the filehandle is closed.
	virtual void OnClose() = 0;

	//! (abstract) Writes data.
	virtual bool Send(unsigned char *data, int len);
	//! (abstract) Writes a string.
	virtual bool Send(const std::string &str);

	//! Returns the file handle.
	int GetFileHandle() const {return mFileHandle;}
	//! Returns whether the IO is open.
	int IsOpen() const {return (mFileHandle>=0);}

	//! Sets the input log file. A copy of all input is saved in this file. If the string is empty, no log is saved.
	bool SetInputLogFile(const std::string &file, bool append=true);
	//! Sets the output log file. A copy of all output is saved in this file. If the string is empty, no log is saved.
	bool SetOutputLogFile(const std::string &file, bool append=true);
};

#endif
