// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "MainLoop.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include "EventClock.h"
#define THISCLASS MainLoop

THISCLASS::MainLoop():
		mQuit(false), mExitCode(0),
		mIOInterfaceList(0), mEventInterfaceList(0) {

	mIOInterfaceList=new IOInterfaceList();
	mEventInterfaceList=new EventInterfaceList();
}

THISCLASS::~MainLoop() {
	delete mIOInterfaceList;
	delete mEventInterfaceList;
}

int THISCLASS::Run() {
	// Reset the quit flag
	mQuit=false;

	// Call event
	OnInitialize();

	// Run loop
	//std::cout << "entering loop" << std::endl;
	while (! mQuit) {
		// Initial empty set of filehandles
		fd_set fhset;
		FD_ZERO(&fhset);
		int maxfh=0;

		// Add all IOInterface objects
		IOInterface *curio=mIOInterfaceList->GetIOInterfaceFirst();
		while (curio) {
			int fh=curio->GetFileHandle();
			if (fh>=0) {
				FD_SET(fh, &fhset);
				if (maxfh<fh) {maxfh=fh;}
			}
			curio=curio->GetIOInterfaceNext();
		}

		// Determine the timeout
		EventClock::tTime nexteventtimeout=mEventInterfaceList->GetNextEventTimeout();
		struct timeval timeout;
		timeout.tv_sec=(int)floor(nexteventtimeout/1000000);
		timeout.tv_usec=nexteventtimeout-timeout.tv_sec*1000000;

		// Wait
		//printf("Wait ... (timeout %d sec %d usec)\n", timeout.tv_sec, timeout.tv_usec);
		//printf(".");
		maxfh++;
		//std::cout << "select: sec=" << timeout.tv_sec << ", usec=" << timeout.tv_usec << std::endl;
		int res = select(maxfh, &fhset, NULL, NULL, &timeout);

		// Update the clock to the current time
		mEventInterfaceList->UpdateClock();

		// Call event
		OnBeforeProcessing();

		// Process IO
		if (res>0) {
			// Check all groups of IOInterface objects
			IOInterface *curio=mIOInterfaceList->GetIOInterfaceFirst();
			while (curio) {
				int fh=curio->GetFileHandle();
				if ((fh>=0) && (FD_ISSET(fh, &fhset))) {
					FD_CLR(fh, &fhset);
					curio->OnInputAvailable();
				}
				curio=curio->GetIOInterfaceNext();
			}
		}

		// Dispatch events
		EventInterface *curevent=mEventInterfaceList->GetEventInterfaceFirst();
		while (curevent) {
			curevent->DispatchEvent();
			curevent=curevent->GetEventInterfaceNext();
		}

		// Call event
		OnAfterProcessing();
	}

	// Call event
	OnTerminate();
	
	// Return the exit code
	return mExitCode;
}

void THISCLASS::Quit() {
	mQuit=true;
}

void THISCLASS::Report(const std::string &name, std::ostream &out, int ccindent) const {
	std::string indent(ccindent*2, ' ');
	std::cout << indent << name << " = MainLoop " << this << std::endl;
	std::cout << indent << "  mQuit = " << mQuit << std::endl;
	std::cout << indent << "  mExitCode = " << mExitCode << std::endl;
	mIOInterfaceList->Report("mIOInterfaceList", out, ccindent+1);
	mEventInterfaceList->Report("mEventInterfaceList", out, ccindent+1);
}
