// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMainLoop
#define classMainLoop

#include "IOInterfaceList.h"
#include "EventInterfaceList.h"

//! A typical main loop.
/*!
	This abstract class implements a main loop for a typical application. It listens to all handles of the attached IOInterfaceList and waits for all timeouts of the attached EventInterfaceList.

	To use this class, derive your main application class from it and overwrite the methods OnInitialize() and OnTerminate(). Your application should first add all EventInterface to the EventInterfaceList and all IOInterfaces to the IOInterfaceList. It may then call Run() to execute the main loop. Immediately before entering the main loop, OnInitialize() is called. Then, the program waits until
		- one of the IOs becomes active (the corresponding IOInterface is invoked)
		- a timeout occurs (the corresponding EventInterface is called)

	Any such event may force the main loop to quit by calling Quit(). The main loop then calls OnTerminate() and returns the exit code (set by SetExitCode).
*/
class MainLoop {

protected:
	//! Set to true to leave the main loop.
	bool mQuit;
	//! The exit code.
	int mExitCode;

	//! The IOInterface list.
	IOInterfaceList *mIOInterfaceList;
	//! The EventInterface list.
	EventInterfaceList *mEventInterfaceList;

public:
	//! Constructor.
	MainLoop();
	//! Destructor.
	~MainLoop();

	//! This executes the main loop until Quit() is called.
	int Run();
	//! Leaves the main loop.
	void Quit();
	//! Sets the exit code.
	void SetExitCode(int code) {mExitCode=code;}
	//! Returns the exit code.
	int GetExitCode() const {return mExitCode;}

	//! (abstract) This method is called right before entering the main loop.
	virtual void OnInitialize() = 0;
	//! (abstract) This method is called right after leaving the main loop.
	virtual void OnTerminate() = 0;
	//! (abstract) This method is called before the events are processed (after waiting for events).
	virtual void OnBeforeProcessing() {}
	//! (abstract) This method is called after the events are processed (before waiting for events).
	virtual void OnAfterProcessing() {}

	//! Returns the IOInterfaceList.
	IOInterfaceList *GetIOInterfaceList() const {return mIOInterfaceList;}
	//! Returns the EventInterfaceList.
	EventInterfaceList *GetEventInterfaceList() const {return mEventInterfaceList;}

	//! Writes a report of this object to the given stream. This is mainly useful for debugging.
	void Report(const std::string &name, std::ostream &out, int ccindent=0) const;

};

#endif
