// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "MipBus.h"
#define	THISCLASS MipBus
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sstream>
#include "termioscms.h"

THISCLASS::MipBus(IOInterfaceList *iolist, EventInterfaceList *evlist, const std::string &device, int baudrate):
		IOInterfaceSerial(iolist, device, baudrate),
		EventInterface(evlist),
		mCurrentMipCommand(0), mTimeout(0), mNextMipDeviceIndex(0),
		mMipLogInterface(0) {

	SetTermiosLocalMode(0);
	for (int i=0; i<mMipDeviceMax; i++) {
		mMipDevice[i]=0;
	}
}

void THISCLASS::OnEvent() {
}

bool THISCLASS::AddMipDevice(MipDevice *mip) {
	int devid=mip->GetDeviceID();
	if (mMipDevice[devid]) {return false;}
	mMipDevice[devid]=mip;
	return true;
}

bool THISCLASS::RemoveMipDevice(MipDevice *mip) {
	int devid=mip->GetDeviceID();
	if (mMipDevice[devid]!=mip) {return false;}
	mMipDevice[devid]=0;
	return true;
}

void THISCLASS::CheckOnline() {
	for (int i=0; i<mMipDeviceMax; i++) {
		if (mMipDevice[i]) {
			if (mMipDevice[i]->IsOpen()) {
				Open();
				return;
			}
		}
	}

	Close();
}

void THISCLASS::OnDataArrival(unsigned char *data, int len) {
	if (len<1) {return;}
	if (mMipLogInterface) {mMipLogInterface->OnDataReceived(this, data, len);}

	// If no current command: skip data
	if (! mCurrentMipCommand) {
		if (mMipLogInterface) {mMipLogInterface->OnUnexpectedDataReceived(this, data, len);}
		return;
	}

	// Get device of current command
	int deviceid=mCurrentMipCommand->GetTxMessage()->GetDestination();
	MipDevice *device=mMipDevice[deviceid];
	if (! device) {
		// The current command belongs to an unknown device. Was the device reconfigured/removed in between? I'll skip the data cancel the command!
		CancelCommand();
		return;
	}

	// If nothing received yet: first byte is direct ACK from MIP
	if (mCurrentMipCommand->GetStatus()==MipCommand::sSent) {
		mCurrentMipCommand->SetStatus(MipCommand::sAckReceived);
		mCurrentMipCommand->SetAck(data[0]);
		device->OnAckReceived(mCurrentMipCommand);
		data++;
		len--;
	}

	// Read message
	MipRxMessage *msg=mCurrentMipCommand->GetRxMessage();
	if (! msg) {
		msg=new MipRxMessage();
		mCurrentMipCommand->SetRxMessage(msg);
	}
	msg->AddReceivedData(data, len);
	if (msg->IsErronous()) {
		mCurrentMipCommand->SetStatus(MipCommand::sErronous);
		device->OnErronousPacketReceived(mCurrentMipCommand);
	} else if (msg->IsComplete()) {
		mCurrentMipCommand->SetStatus(MipCommand::sReceived);
		device->OnPacketReceived(mCurrentMipCommand);
	} else {
		return;
	}

	// Next command
	delete mCurrentMipCommand;
	mCurrentMipCommand=0;
	SendNextCommand();
}

void THISCLASS::OnBeforeClose() {
	if (mMipLogInterface) {mMipLogInterface->OnBeforeClose(this);}
}

void THISCLASS::OnAfterOpen() {
	if (mMipLogInterface) {mMipLogInterface->OnAfterOpen(this);}
}

void THISCLASS::OnAfterOpenError() {
	if (mMipLogInterface) {mMipLogInterface->OnAfterOpenError(this);}
}

bool THISCLASS::GetNextCommand(int from, int to) {
	while (from<to) {
		if (mMipDevice[from]) {
			mCurrentMipCommand=mMipDevice[from]->GetNextCommand();
			if (mCurrentMipCommand) {
				mNextMipDeviceIndex=from+1;
				return true;
			}
		}

		from++;
	}

	return 0;
}

void THISCLASS::SendNextCommand() {
	if (mCurrentMipCommand) {return;}
	if (mFileHandle<0) {return;}

	// Get next mip command
	if (! GetNextCommand(mNextMipDeviceIndex, mMipDeviceMax)) {
		GetNextCommand(0, mNextMipDeviceIndex);
		if (! mCurrentMipCommand) {return;}
	}

	MipTxMessage *msg=mCurrentMipCommand->GetTxMessage();
	if (msg==0) {
		delete mCurrentMipCommand;
		mCurrentMipCommand=0;
		return SendNextCommand();
	}

	// Get packet
	unsigned char *packet=msg->GetPacket();
	int len=msg->GetLength();
	if (mMipLogInterface) {mMipLogInterface->OnSendPacket(this, packet, len);}

	//std::cout << "Sending packet: start (fh " << mFileHandle << ")" << std::endl;
	struct termios tio;
	bzero(&tio, sizeof(tio));
	tcgetattr(mFileHandle, &tio);
	//std::cout << "Sending packet: tcgetattr done" << std::endl;

	// Send first byte with mark parity
	tio.c_cflag |= PARENB;
	tio.c_cflag |= CMSPAR;
	tio.c_cflag |= PARODD;
	tcsetattr(mFileHandle, TCSADRAIN, &tio);
	//std::cout << "Sending packet: tcsetattr 1 done" << std::endl;
	write(mFileHandle, packet, 1);
	//std::cout << "Sending packet: write 1 done" << std::endl;

	// Send the rest with space parity
	tio.c_cflag &= ~PARODD;
	tcsetattr(mFileHandle, TCSADRAIN, &tio);
	//std::cout << "Sending packet: tcsetattr 2 done" << std::endl;
	write(mFileHandle, packet+1, len-1);
	//std::cout << "Sending packet: write 2 done" << std::endl;

	mCurrentMipCommand->SetStatus(MipCommand::sSent);
	return;
}

#ifdef USE_CMSPAR
void THISCLASS::SendPacket(unsigned char *packet, int len) const {
	struct termios tio;
	bzero(&tio, sizeof(tio));
	tcgetattr(mFileHandle, &tio);

	// Send first byte with mark parity
	tio.c_cflag |= PARENB;
	tio.c_cflag |= CMSPAR;
	tio.c_cflag |= PARODD;
	tcsetattr(mFileHandle, TCSADRAIN, &tio);
	write(mFileHandle, packet, 1);

	// Send the rest with space parity
	tio.c_cflag &= ~PARODD;
	tcsetattr(mFileHandle, TCSADRAIN, &tio);
	write(mFileHandle, packet+1, len-1);
}
#else
void THISCLASS::SendPacket(unsigned char *packet, int len) const {
	// The parity matrix (0 = even number of 1's)
	static const int parity[256]={
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0};

	struct termios tio;
	bzero(&tio, sizeof(tio));
	tcgetattr(mFileHandle, &tio);

	// Send first byte with mark parity
	tio.c_cflag |= PARENB;
	if (parity[packet[0]]==0) {
		tio.c_cflag |= PARODD;
	} else {
		tio.c_cflag &= ~PARODD;
	}
	tcsetattr(mFileHandle, TCSADRAIN, &tio);
	write(mFileHandle, packet, 1);

	// Send the rest (byte by byte) with space parity
	for (int i=1; i<len; i++) {
		if (parity[packet[i]]==0) {
			tio.c_cflag &= ~PARODD;
		} else {
			tio.c_cflag |= PARODD;
		}
		tcsetattr(mFileHandle, TCSADRAIN, &tio);
		write(mFileHandle, &packet[i], 1);
	}
}
#endif

void THISCLASS::CancelCommand() {
	if (mCurrentMipCommand) {
		delete mCurrentMipCommand;
		mCurrentMipCommand=0;
	}

	SendNextCommand();
}

void THISCLASS::CancelAllCommands() {
	for (int i=0; i<mMipDeviceMax; i++) {
		if (mMipDevice[i]) {mMipDevice[i]->FlushCommandQueue();}
	}

	CancelCommand();
}

void THISCLASS::Report(const std::string &name, std::ostream &out, int ccindent) const {
	std::string indent(ccindent*2, ' ');
	std::cout << indent << name << " = MipBus " << this << std::endl;
	std::cout << indent << "  mFileHandle = " << mFileHandle << std::endl;
	std::cout << indent << "  mNextMipDeviceIndex = " << mNextMipDeviceIndex << std::endl;
	std::cout << indent << "  mMipLogInterface = MipLogInterface " << mMipLogInterface << std::endl;

	for (int i=0; i<mMipDeviceMax; i++) {
		if (mMipDevice[i]) {
			std::ostringstream name;
			name << "mDevice[" << i << "]";
			mMipDevice[i]->Report(name.str(), out, ccindent+1);
		}
	}
}
