// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipBus
#define classMipBus

class MipBus;

#include <string>
#include "IOInterfaceSerial.h"
#include "MipMessage.h"
#include "MipDevice.h"
#include "MipCommand.h"
#include "MipLogInterface.h"
#include "EventInterface.h"

//! Models a Mip bus
/*!
	A Mip bus contains several MipDevice objects with are identified by their device ID.

	Due to protocol restrictions (to avoid collisions), only one command can be sent over the same serial port (MIP bus) at a time. The next command can be sent only after a reply for the current command has been received. Therefore, commands are queued in the MipDevices. If several MipDevices are attached to the same MipBus, they are served in a round robin fashion.

	The queue length is currently limited to 16 commands. Further MipCommands are silently discarded.
*/
class MipBus: public IOInterfaceSerial, public EventInterface {
friend class MipManager;

protected:
	//! Maximum number of MipDevice objects.
	static const int mMipDeviceMax=128;
	//! Mip devices on the bus.
	MipDevice *mMipDevice[mMipDeviceMax];

	//! The current Mip command.
	MipCommand *mCurrentMipCommand;
	//! Timeout in milliseconds.
	int mTimeout;
	//! The next mip device to ask for a mip comand.
	int mNextMipDeviceIndex;

	//! The log interface.
	MipLogInterface *mMipLogInterface;

	//! Asks a set of MipDevice objects for a new command and sets it as the current command if found.
	bool GetNextCommand(int from, int to);

	//! Sends a packet.
	void SendPacket(unsigned char *packet, int len) const;

public:
	//! Constructor.
	MipBus(IOInterfaceList *iolist, EventInterfaceList *evlist, const std::string &device, int baudrate=19200);
	
	// Overwritten IOInterface methods
	virtual void OnDataArrival(unsigned char *data, int len);
	virtual void OnClose() {}

	// Overwritten IOInterfaceSerial methods
	virtual void OnBeforeClose();
	virtual void OnAfterOpen();
	virtual void OnAfterOpenError();

	// Overwritten EventInterface methods
	void OnEvent();

	//! Sets the mip log interface.
	void SetLogInterface(MipLogInterface *miplog) {mMipLogInterface=miplog;}
	//! Returns the mip log interface.
	MipLogInterface *GetLogInterface() {return mMipLogInterface;}

	//! Returns the maximum number of MipBus objects.
	int GetMipDeviceMax() {return mMipDeviceMax;}
	//! Returns a Mip device.
	MipDevice *GetMipDevice(int i) {return mMipDevice[i];}
	//! Adds a Mip device.
	bool AddMipDevice(MipDevice *mip);
	//! Removes a Mip device.
	bool RemoveMipDevice(MipDevice *mip);
	//! Returns whether the bus is idle (no pending command).
	bool IsIdle() {return mCurrentMipCommand==0;}
	//! Returns the current command (or 0 if the bus is idle).
	MipCommand *GetCurrentCommand() {return mCurrentMipCommand;}
	//! Sends the next command in the queue (if the bus is free and the queue is not empty).
	void SendNextCommand();
	//! Aborts the current command.
	void CancelCommand();
	//! Aborts the current command and flushes the command queue.
	void CancelAllCommands();

	//! Checks the online status and opens or closes the serial port if necessary.
	void CheckOnline();

	//! Writes a report of this object to the given stream. This is mainly useful for debugging.
	void Report(const std::string &name, std::ostream &out, int ccindent=0) const;
};

#endif
