// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipCommand
#define classMipCommand

class MipCommand;

#include "MipTxMessage.h"
#include "MipRxMessage.h"
#include "MipBus.h"

//! MipBus Protocol Command.
/*!
	The MipCommand class is the abstraction used when sending a command to a MIP device. It mainly consists of a MipTxMessage (the message transmitted to the MIP) and a MipRxMessage (the reply received from the MIP). Furthermore, it contains a comman ID which not transmitted to the MIP.

	A MIP command may be in various states (eStatus). The most important states are:
		- sQueued: The command is in the queue, waiting for being transmitted.
		- sNotSent: The command has been dequeued, but sending has not completed yet.
		- sSent: The command has been sent but no reply has been received yet.
		- sAckReceived: An ack was received but the complete reply is still awaited.
		- sReceived: A complete and correct reply has been received.
		- sErronous: An erroneous reply has been received (wrong CRC).
		- sCancelled: The command has been cancelled.
*/
class MipCommand {
friend class MipBus;

public:
	//! Status flags.
	enum eStatus {
		sNone,
		sQueued,
		sNotSent,
		sSent,
		sAckReceived,
		sReceived,
		sErronous,
		sCancelled
	};

private:
	//! A number associated with the command. This can be freely chosen by the application programmer to distinguish commands.
	int mID;
	//! The associated MipDevice object.
	MipDevice *mMipDevice;
	//! The transmitted message (or the message to transmit).
	MipTxMessage *mTxMessage;
	//! The received message.
	MipRxMessage *mRxMessage;
	//! The command status.
	eStatus mStatus;
	//! The acknoledgement flag that is sent by the MIP after having received the message data.
	unsigned char mAck;

public:
	//! Constructor.
	MipCommand(int cmdid, MipTxMessage *txmsg);
	//! Destructor.
	~MipCommand();

	//! Returns the command ID.
	int GetID() const {return mID;}
	//! Returns the message to send.
	MipRxMessage *GetRxMessage() const {return mRxMessage;}
	//! Returns the received message.
	MipTxMessage *GetTxMessage() const {return mTxMessage;}
	//! Returns the MipBus.
	MipDevice *GetMipDevice() const {return mMipDevice;}
	//! Returns the status of the command.
	eStatus GetStatus() const {return mStatus;}
	//! Returns the status of the command.
	unsigned char GetAck() const {return mAck;}

	//! Sets the command ID.
	void SetCommandID(int id);
	//! Sets the message to send.
	void SetRxMessage(MipRxMessage *msg);
	//! Sets the received message.
	void SetTxMessage(MipTxMessage *msg);
	//! Sets the MipBus.
	void SetMipDevice(MipDevice *mipdevice);
	//! Sets the status of the command.
	void SetStatus(eStatus status);
	//! Sets the ack of the command.
	void SetAck(unsigned char ack);
};

#endif
