// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "MipDevice.h"
#include "Constants.h"
#define	THISCLASS MipDevice

THISCLASS::MipDevice(MipBus *mipbus, int deviceid):
		mMipBus(mipbus), mDeviceID(deviceid),
		mMipCommandExecuteIndex(0), mMipCommandAddIndex(0) {

	if (mipbus) {mipbus->AddMipDevice(this);}

	for (int i=0; i<mMipCommandMax; i++) {
		mMipCommand[i]=0;
	}
}

THISCLASS::MipDevice(MipManager *mm, const std::string &device, int baudrate, int deviceid):
		mMipBus(0), mDeviceID(0),
		mMipCommandExecuteIndex(0), mMipCommandAddIndex(0) {

	for (int i=0; i<mMipCommandMax; i++) {
		mMipCommand[i]=0;
	}

	SetDevice(mm, device, baudrate, deviceid);
}

void THISCLASS::SetMipBus(MipBus *mipbus) {
	if (mMipBus==mipbus) {return;}
	if (mMipBus) {
		mMipBus->RemoveMipDevice(this);
		mMipBus->CheckOnline();
	}
	mMipBus=mipbus;
	if (mMipBus) {
		mMipBus->AddMipDevice(this);
		mMipBus->CheckOnline();
	}
}

void THISCLASS::SetDeviceID(int deviceid) {
	if (mDeviceID==deviceid) {return;}
	if (mMipBus) {
		mMipBus->RemoveMipDevice(this);
	}
	mDeviceID=deviceid;
	if (mMipBus) {
		mMipBus->AddMipDevice(this);
		mMipBus->CheckOnline();
	}
}

void THISCLASS::SetDevice(MipManager *mm, const std::string &device, int baudrate, int deviceid) {
	MipBus *mipbus=mm->FindMipBus(device);
	if (mipbus) {mipbus->SetBaudRate(baudrate);}
	SetDevice(mipbus, deviceid);
}

void THISCLASS::SetDevice(MipBus *mipbus, int deviceid) {
	if (mMipBus) {mMipBus->RemoveMipDevice(this);}
	if (mMipBus!=mipbus) {
		if (mMipBus) {mMipBus->CheckOnline();}
		mMipBus=mipbus;
	}
	mDeviceID=deviceid;
	if (mMipBus) {
		mMipBus->AddMipDevice(this);
		mMipBus->CheckOnline();
	}
}

bool THISCLASS::Open() {
	if (mIsOpen) {return true;}
	mIsOpen=true;
	if (mMipBus) {mMipBus->CheckOnline();}
	return true;
}

bool THISCLASS::Close() {
	if (! mIsOpen) {return true;}
	mIsOpen=false;
	if (mMipBus) {mMipBus->CheckOnline();}
	return true;
}

void THISCLASS::OnAckReceived(MipCommand *cmd) {
}

void THISCLASS::OnPacketReceived(MipCommand *cmd) {
	int cmdid=cmd->GetID();
	MipRxMessage *rxmsg=cmd->GetRxMessage();
	MipTxMessage *txmsg=cmd->GetTxMessage();
	MipMessage::eType rxtype=rxmsg->GetType();
	MipMessage::eType txtype=txmsg->GetType();

	if (rxtype==MipMessage::sReadError) {
		int code=rxmsg->ReadSignedInteger16();
		OnReceivedReadError(cmdid, (eError)code);
	} else if (rxtype==MipMessage::sClearError) {
		OnReceivedClearError(cmdid);
	} else if (rxtype==MipMessage::sReadWarning) {
		int code=rxmsg->ReadSignedInteger16();
		OnReceivedReadWarning(cmdid, (eWarning)code);
	} else if (rxtype==MipMessage::sClearWarning) {
		OnReceivedClearWarning(cmdid);
	} else if (rxtype==MipMessage::sReadAxisStatus) {
		int status=rxmsg->ReadSignedInteger16();
		OnReceivedReadAxisStatus(cmdid, (eAxisStatusFlags)status);
	} else if (rxtype==MipMessage::sReadVersion) {
		int command_interpreter_version=rxmsg->ReadSignedInteger16();
		int runtime_library_version=rxmsg->ReadSignedInteger16();
		int hardware_revision=rxmsg->ReadSignedInteger16();
		int bootrom_version=rxmsg->ReadSignedInteger16();
		int application_number=rxmsg->ReadSignedInteger16();
		int application_version=rxmsg->ReadSignedInteger16();
		OnReceivedReadVersion(cmdid, command_interpreter_version, runtime_library_version, hardware_revision, bootrom_version, application_number, application_version);
	} else if (rxtype==MipMessage::sSetPosVelocity) {
		OnReceivedSetPosVelocity(cmdid);
	} else if (rxtype==MipMessage::sSetWaitMode) {
		OnReceivedSetWaitMode(cmdid);
	} else if (rxtype==MipMessage::sMoveAbsolute) {
		OnReceivedMoveAbsolute(cmdid);
	} else if (rxtype==MipMessage::sMoveRelative) {
		OnReceivedMoveRelative(cmdid);
	} else if (rxtype==MipMessage::sStopMotion) {
		OnReceivedStopMotion(cmdid);
	} else if (rxtype==MipMessage::sFindHomeSys) {
		OnReceivedFindHomeSys(cmdid);
	} else if (rxtype==MipMessage::sReadPosition) {
		int value=rxmsg->ReadSignedInteger32();
		OnReceivedReadPosition(cmdid, value);
	} else if (rxtype==MipMessage::sReadVelocity) {
		int value=rxmsg->ReadSignedInteger32();
		OnReceivedReadVelocity(cmdid, value);
	} else if (rxtype==MipMessage::sReadVelocityMean) {
		int value=rxmsg->ReadSignedInteger32();
		OnReceivedReadVelocityMean(cmdid, value);
	} else if (rxtype==MipMessage::sReadCurrent) {
		int value=rxmsg->ReadSignedInteger32();
		OnReceivedReadCurrent(cmdid, value);
	} else if (rxtype==MipMessage::sCheckCRC) {
		bool ok=rxmsg->ReadBool();
		int crc=rxmsg->ReadUnsignedInteger16();
		OnReceivedCheckCRC(cmdid, ok, crc);
	}
}

void THISCLASS::OnErronousPacketReceived(MipCommand *cmd) {
}

MipCommand *THISCLASS::GetNextCommand() {
	MipCommand *cmd=mMipCommand[mMipCommandExecuteIndex];
	if (! cmd) {return 0;}

	mMipCommand[mMipCommandExecuteIndex]=0;
	cmd->SetStatus(MipCommand::sNotSent);

	mMipCommandExecuteIndex++;
	if (mMipCommandExecuteIndex>=mMipCommandMax) {mMipCommandExecuteIndex=0;}
	return cmd;
}

bool THISCLASS::AddCommand(MipCommand *cmd) {
	if (mMipCommand[mMipCommandAddIndex]) {return false;}
	cmd->SetMipDevice(this);
	mMipCommand[mMipCommandAddIndex]=cmd;
	mMipCommandAddIndex++;
	if (mMipCommandAddIndex>=mMipCommandMax) {mMipCommandAddIndex=0;}
	if (mMipBus) {mMipBus->SendNextCommand();}
	return true;
}

void THISCLASS::FlushCommandQueue() {
	for (int i=0; i<mMipCommandMax; i++) {
		if (mMipCommand[i]) {
			delete mMipCommand[i];
			mMipCommand[i]=0;
		}
	}
	mMipCommandAddIndex=0;
	mMipCommandExecuteIndex=0;
}

bool THISCLASS::SendCommand(int cmdid, MipMessage::eType type) {
	MipTxMessage *msg=new MipTxMessage(0, mDeviceID, type);
	return AddCommand(new MipCommand(cmdid, msg));
}

bool THISCLASS::SendCommand_SignedInteger32(int cmdid, MipMessage::eType type, int param1) {
	MipTxMessage *msg=new MipTxMessage(0, mDeviceID, type);
	msg->WriteSignedInteger32(param1);
	return AddCommand(new MipCommand(cmdid, msg));
}

bool THISCLASS::SendCommand_Float(int cmdid, MipMessage::eType type, float param1) {
	MipTxMessage *msg=new MipTxMessage(0, mDeviceID, type);
	msg->WriteFloat(param1);
	return AddCommand(new MipCommand(cmdid, msg));
}

bool THISCLASS::SendCommand_Byte(int cmdid, MipMessage::eType type, unsigned char param1) {
	MipTxMessage *msg=new MipTxMessage(0, mDeviceID, type);
	msg->WriteByte(param1);
	return AddCommand(new MipCommand(cmdid, msg));
}

bool THISCLASS::SendCommand_ByteFloatFloat(int cmdid, MipMessage::eType type, unsigned char param1, float param2, float param3) {
	MipTxMessage *msg=new MipTxMessage(0, mDeviceID, type);
	msg->WriteByte(param1);
	msg->WriteFloat(param2);
	msg->WriteFloat(param3);
	return AddCommand(new MipCommand(cmdid, msg));
}

void THISCLASS::Report(const std::string &name, std::ostream &out, int ccindent) const {
	std::string indent(ccindent*2, ' ');
	std::cout << indent << name << " = MipBus " << this << std::endl;
	std::cout << indent << "  mDeviceID = " << mDeviceID << std::endl;
	std::cout << indent << "  mIsOpen = " << mIsOpen << std::endl;
	std::cout << indent << "  mMipBus = MipBus " << mMipBus << "" << std::endl;
	std::cout << indent << "  mMipCommandExecuteIndex = " << mMipCommandExecuteIndex << std::endl;
	std::cout << indent << "  mMipCommandAddIndex = " << mMipCommandAddIndex << std::endl;

	for (int i=0; i<mMipCommandMax; i++) {
		if (mMipCommand[i]) {
			std::cout << indent << "  mMipCommand[" << i << "] = MipCommand " << mMipCommand[i];
			MipTxMessage *txm=mMipCommand[i]->GetTxMessage();
			if (txm) {
				std::cout << ": Type=" << (int)(txm->GetType()) << std::endl;
			}
		}
	}
}
