// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipDevice
#define classMipDevice

class MipDevice;

#include <string>
#include "MipBus.h"
#include "MipCommand.h"
#include "MipManager.h"

//! Models a Mip device on a Mip bus.
/*!
	A MipDevice objects represents one physical MIP. A MIP is addressed with a device ID on a serial bus. The serial bus is modelled with a MipBus object and the MipManager handles a list of MipBus objects.

	This class should not be used directly. It is intended to be inherited by a subclass implementing the desired functionality. MipDevice objects can then be created as follows:
		- MipManager *mm=new MipManager();
		- MyMip *md1=MyMip(mm, "/dev/ttyS0", 4800, 1)
		- MyMip *md2=MyMip(mm, "/dev/ttyS0", 4800, 2)
	This creates two MyMip objects (which inherit from the MipDevice class) on the same serial interface but with a different device ID. The MipManager automatically creates a MipBus object for the serial interface "/dev/ttyS0" and attaches the two MyMip objects with their respective device IDs.
*/
class MipDevice {

protected:
	#include "MipErrors.inc.h"
	#include "MipWarnings.inc.h"
	#include "MipAxisStatus.inc.h"
	#include "MipTrajectoryFlags.inc.h"

private:
	//! The attached MipBus object which is used to communicate.
	MipBus *mMipBus;
	//! The device ID (slave).
	int mDeviceID;
	//! Tells whether the device is (virtually) open. If all devices on a bus are closed, the serial port of the bus is closed.
	bool mIsOpen;

	//! Maximum number of queued commands.
	static const int mMipCommandMax=16;
	//! Waiting Mip commands.
	MipCommand *mMipCommand[mMipCommandMax];
	//! Index of the next Mip command to execute.
	int mMipCommandExecuteIndex;
	//! Index of the next free slot for new commands.
	int mMipCommandAddIndex;
	
public:
	//! Constructor.
	MipDevice(MipBus *mipbus=0, int deviceid=0);
	//! Constructor.
	MipDevice(MipManager *mm, const std::string &device, int baudrate, int deviceid);

	//! Returns the MipBus object.
	MipBus *GetMipBus() const {return mMipBus;}
	//! Returns the device id (slave).
	int GetDeviceID() const {return mDeviceID;}

	//! Opens the MipDevice. The corresponding MipBus opens the serial interface automatically if necessary.
	bool Open();
	//! Closes the MipDevice. The corresponding MipBus closes the serial interface automatically if no other MipDevice is open on this MipBus.
	bool Close();
	//! Returns whether the MipDevice is open.
	bool IsOpen() {return mIsOpen;}

	//! Called when an ACK is received. The MIP sends an ACK after having received the message and checked its checksum.
	void OnAckReceived(MipCommand *cmd);
	//! Called when the response packet has been received successfully.
	void OnPacketReceived(MipCommand *cmd);
	//! Called when the response packet has been received erronously (wrong packet checksum, wrong length checksum).
	void OnErronousPacketReceived(MipCommand *cmd);

	//! Adds a command to the queue.
	bool AddCommand(MipCommand *cmd);
	//! Adds a command to the beginning of the queue. It will be executed just after the current
	bool AddPriorityCommand(MipCommand *cmd);
	//! Deletes the command queue. This does not abort the current command.
	void FlushCommandQueue();
	//! Switches to the next command.
	MipCommand *GetNextCommand();
	//! Returns the maximum number of commands.
	int GetMipCommandMax() {return mMipCommandMax;}
	//! Returns a MipCommand.
	MipCommand *GetMipCommand(int i) {return mMipCommand[i];}
	
	//! Sets the MipBus object.
	void SetMipBus(MipBus *MipBus);
	//! Sets the device ID (slave).
	void SetDeviceID(int deviceid);
	//! Sets the mip bus and the device id. The mip bus is determined using the MipManager object.
	void SetDevice(MipManager *mm, const std::string &device, int baudrate, int deviceid);
	//! Sets the mip bus and the device id.
	void SetDevice(MipBus *mipbus, int deviceid);

	bool SendCommand(int cmdid, MipMessage::eType type);
	bool SendCommand_SignedInteger32(int cmdid, MipMessage::eType type, int param1);
	bool SendCommand_Float(int cmdid, MipMessage::eType type, float param1);
	bool SendCommand_Byte(int cmdid, MipMessage::eType type, unsigned char param1);
	bool SendCommand_ByteFloatFloat(int cmdid, MipMessage::eType type, unsigned char param1, float param2, float param3);

	bool SendReadAxisStatus(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadAxisStatus);}
	bool SendReadError(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadError);}
	bool SendClearError(int cmdid=0) {return SendCommand(cmdid, MipMessage::sClearError);}
	bool SendReadWarning(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadWarning);}
	bool SendClearWarning(int cmdid=0) {return SendCommand(cmdid, MipMessage::sClearWarning);}
	bool SendReadVersion(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadVersion);}

	bool SendSetRegMode(int cmdid, unsigned char mode) {return SendCommand_Byte(cmdid, MipMessage::sSetRegMode, mode);}
	bool SendSetPosVelocity(int cmdid, float velocity) {return SendCommand_Float(cmdid, MipMessage::sSetPosVelocity, velocity);}
	bool SendSetWaitMode(int cmdid, unsigned char mode) {return SendCommand_Byte(cmdid, MipMessage::sSetWaitMode, mode);}
	bool SendMoveAbsolute(int cmdid, int position) {return SendCommand_SignedInteger32(cmdid, MipMessage::sMoveAbsolute, position);}
	bool SendMoveRelative(int cmdid, int relposition) {return SendCommand_SignedInteger32(cmdid, MipMessage::sMoveRelative, relposition);}
	bool SendStopMotion(int cmdid=0, unsigned char stopmode=0) {return SendCommand_Byte(cmdid, MipMessage::sStopMotion, stopmode);}
	bool SendFindHomeSys(int cmdid=0, int timeoutms=0) {return SendCommand_SignedInteger32(cmdid, MipMessage::sFindHomeSys, timeoutms);}

	bool SendReadPosition(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadPosition);}
	bool SendReadVelocity(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadVelocity);}
	bool SendReadVelocityMean(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadVelocityMean);}
	bool SendReadCurrent(int cmdid=0) {return SendCommand(cmdid, MipMessage::sReadCurrent);}

	bool SendCheckCRC(int cmdid, unsigned char selection) {return SendCommand_Byte(cmdid, MipMessage::sCheckCRC, selection);}

	virtual void OnReceivedReadAxisStatus(int cmdid, eAxisStatusFlags status) = 0;
	virtual void OnReceivedReadError(int cmdid, eError error) = 0;
	virtual void OnReceivedClearError(int cmdid) = 0;
	virtual void OnReceivedReadWarning(int cmdid, eWarning warning) = 0;
	virtual void OnReceivedClearWarning(int cmdid) = 0;
	virtual void OnReceivedReadVersion(int cmdid, int command_interpreter_version, int runtime_library_version, int hardware_revision, int bootrom_version, int application_number, int application_version) = 0;
	virtual void OnReceivedSetPosVelocity(int cmdid) = 0;
	virtual void OnReceivedSetWaitMode(int cmdid) = 0;
	virtual void OnReceivedMoveAbsolute(int cmdid) = 0;
	virtual void OnReceivedMoveRelative(int cmdid) = 0;
	virtual void OnReceivedStopMotion(int cmdid) = 0;
	virtual void OnReceivedFindHomeSys(int cmdid) = 0;
	virtual void OnReceivedReadPosition(int cmdid, int value) = 0;
	virtual void OnReceivedReadVelocity(int cmdid, int value) = 0;
	virtual void OnReceivedReadVelocityMean(int cmdid, int value) = 0;
	virtual void OnReceivedReadCurrent(int cmdid, int value) = 0;
	virtual void OnReceivedCheckCRC(int cmdid, bool ok, int crc) = 0;

	//! Writes a report of this object to the given stream. This is mainly useful for debugging.
	void Report(const std::string &name, std::ostream &out, int ccindent=0) const;
};

#endif
