// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipLogInterface
#define classMipLogInterface

class MipLogInterface;

#include <string>
#include "MipManager.h"
#include "MipBus.h"
#include "MipDevice.h"

//! Receiver of log messages from the MipManager object and the attached MipBus objects.
/*!
	This is an abstract interface intended for logging. It allows to catch various internal events of the MipManager and the MipBus objects.

	To use this interface, create a subclass of it and implement its abstract methods. Attach your class to the MipManager using:
		- mymipmanager->SetLogInterface(mymiplogclass);
	It is recommended to do this immediately after creating the MipManager object.
*/
class MipLogInterface {

protected:
	//! The associated MipManager object.
	MipManager *mMipManager;

	//! Turns a piece of data (e.g. a packet) into hexadecimal representation.
	std::string DataToString(unsigned char *packet, int len);

public:
	//! Constructor.
	MipLogInterface(MipManager *mipmanager);

	//! Sets the MipManager. This method is automatically invoked when mipmanager->SetLogInterface is called.
	void SetMipManager(MipManager *mipmanager) {mMipManager=mipmanager;}
	//! Returns the MipManager.
	MipManager *GetMipManager() {return mMipManager;}

	//! (abstract) Called when data is received.
	virtual void OnDataReceived(MipBus *mipbus, unsigned char *data, int len) = 0;
	//! (abstract) Called when unexpected data is received. That means, some data is received without having issued a command. This may happen if another device is on the same bus or if a Mip is executing some command when it is plugged in.
	virtual void OnUnexpectedDataReceived(MipBus *mipbus, unsigned char *data, int len) = 0;
	//! (abstract) Called before a packet is sent.
	virtual void OnSendPacket(MipBus *mipbus, const unsigned char *data, int len) = 0;
	//! (abstract) Called before the serial port of a MipBus is closed.
	virtual void OnBeforeClose(MipBus *mipbus) = 0;
	//! (abstract) Called right after opening the serial port of a MipBus.
	virtual void OnAfterOpen(MipBus *mipbus) = 0;
	//! (abstract) Called if an error occured when opening a serial port.
	virtual void OnAfterOpenError(MipBus *mipbus) = 0;

};

#endif
