// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipManager
#define classMipManager

class MipManager;

#include <string>
#include "MipBus.h"
#include "MipDevice.h"
#include "MipLogInterface.h"
#include "IOInterfaceList.h"
#include "EventInterfaceList.h"

//! Manages a Set of MipBus objects with attached MipDevices.
/*!
	This is the first object to create when working with this library. All MipDevice objects are attached to it.

	The MipManager requires an IOInterfaceList and an EventInterfaceList. If these arguments are not provided, the MipManager creates these lists for you and you can access them using GetIOInterfaceList and GetEventInterfaceList. Note that if you don't use the MainLoop implementation of this library as your program's main loop, you need to listen to all filehandles of the IOInterfaceList and call the OnInputAvailable() method on the corresponding IOInterface object when filehandles become active.

	The MipManager transparently maintains a list of MipBus objects. If you add a MipDevice on a serial port, a MipBus object for that serial port is created (unless such an object exists already) and the MipDevice object is attached to this MipBus. Hence, the structure looks like this:
		- MipManager
			- MipBus /dev/ttyS0
				- MipDevice ID=1
				- MipDevice ID=2
				- ...
			- MipBus /dev/ttyS1
				- MipDevice ID=1
				- ...
			- MipBus ...

	You can use the Report() method to dump the data structure of the MipManager.
*/
class MipManager {

private:
	//! The maximum number of MipBus objects.
	static const int mMipBusMax=8;
	//! The MipBus objects.
	MipBus *mMipBus[mMipBusMax];

	//! The IOInterface list.
	IOInterfaceList *mIOInterfaceList;
	//! The EventInterface list.
	EventInterfaceList *mEventInterfaceList;

	//! The log interface.
	MipLogInterface *mMipLogInterface;

public:
	//! Constructor.
	MipManager(IOInterfaceList *iolist=0, EventInterfaceList *evlist=0);
	//! Destructor.
	~MipManager();

	//! Sets the mip log interface.
	void SetLogInterface(MipLogInterface *miplog);
	//! Returns the mip log interface.
	MipLogInterface *GetLogInterface() {return mMipLogInterface;}
	//! Returns the IO interface list.
	IOInterfaceList *GetIOInterfaceList() {return mIOInterfaceList;}
	//! Returns the event interface list.
	EventInterfaceList *GetEventInterfaceList() {return mEventInterfaceList;}

	//! Returns a MipBus object.
	MipBus *FindMipBus(const std::string &device);

	//! Returns the maximum number of MipBus objects.
	int GetMipBusMax() {return mMipBusMax;}
	//! Returns a MipBus object.
	MipBus *GetMipBus(int i) {return mMipBus[i];}

	//! Writes a report of this object to the given stream. This is mainly useful for debugging.
	void Report(const std::string &name, std::ostream &out, int ccindent=0) const;
};

#endif
