// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipRxMessage
#define classMipRxMessage

#include "MipMessage.h"

//! MipBus Protocol Receive Message.
/*!
	This class handles a received message. It contains methods to check the CRC and the header checksum, as well as methods to read the information of the message, i.e. methods that transform the received bits into integers, floats and other data types.
*/
class MipRxMessage: public MipMessage {

private:
	//! Index of the next data byte to read.
	int mReadIndex;
	//! Index of the next data byte to write.
	int mReceiveIndex;
	//! Whether the packet is erronous (length checksum error or packet checksum error).
	bool mErronous;
	//! Whether the packet has been received completely.
	bool mComplete;
	//! The length of the data in the packet (as read in the header).
	int mDataLength;

public:
	//! Constructor.
	MipRxMessage();

	//! Returns the length.
	int GetLength() const {return mDataLength+7;}
	//! Returns whether the packet is erronous or not. (Erronous means that either the length checksum or the packet checksum are wrong.) An erronous packet is always marked as complete.
	bool IsErronous() const {return mErronous;}
	//! Returns whether the packet is complete or not. (Not complete means that not all of it has been received yet.)
	bool IsComplete() const {return mComplete;}

	//! Adds received data.
	void AddReceivedData(unsigned char *data, int len);

	//! Reads a command (byte) from the data.
	unsigned char ReadCommand() {ReadByte();}
	//! Reads a byte from the data.
	unsigned char ReadByte();
	//! Reads an unsigned 16bit integer from the data.
	unsigned short int ReadUnsignedInteger16();
	//! Reads a signed 16bit integer from the data.
	short int ReadSignedInteger16();
	//! Reads an unsigned 32bit integer from the data.
	unsigned int ReadUnsignedInteger32();
	//! Reads a signed 32bit integer from the data.
	int ReadSignedInteger32();
	//! Reads a float from the data.
	float ReadFloat();
	//! Reads a bool from the data.
	bool ReadBool();

	//! Reads the header and checks the header checksum in the length field. Returns true if the checksum is correct.
	bool ReadHeader();
	//! Reads the length and checks the header checksum in the length field. Returns true if the checksum is correct.
	bool ReadLength();
	//! Checks the packet checksum.
	bool CheckCRC();
};

#endif
