// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "MyMip.h"
#include <mip/MipBus.h>
#define	THISCLASS MyMip

THISCLASS::MyMip(MipManager *mm, const std::string &device, int baudrate, int deviceid): 
		EventInterface(mm->GetEventInterfaceList()),
		MipDevice(mm, device, baudrate, deviceid),
		mDriveMotor(false), mAngleNow(0), mMoving(false), mHome(false),
		mReferenced(false), mError(0), mWarning(0) {

}

void THISCLASS::Initialize() {
	// Open the MIP device
	Open();

	// To start, clear any errors and go home
	SendClearError(0);
	SendClearWarning(0);
	SendSetPosVelocity(0, 10);
	SendFindHomeSys(0);

	// We are ready now
	mDriveMotor=true;

	// Call OnEvent after 1 sec
	SetNextEvent(1000);
}

void THISCLASS::Terminate() {
	// Before quitting, put the motor back to its home position.
	SendFindHomeSys(0);
}

void THISCLASS::SendAngle(double angle) {
	// Don't do anything if the motors are either disabled or not referenced.
	if (! mDriveMotor) {return;}
	if (! mReferenced) {return;}

	// Translate the angle into a motor position.
	int position=AngleToPosition(angle);
	SendMoveAbsolute(0, position);
}

void THISCLASS::OnEvent() {
	// Calculate the required angle.
	// The code here is just a demonstration. It uses the seconds of computer clock to
	// to steer the motor.
	time_t curtime=time(0);
	struct tm *tim=localtime(&curtime);
	int seconds=tim->tm_sec;
	double angle=PI/30*seconds;
	if (angle>PI) {angle=2*PI-angle;}
	std::cout << "Set position: " << angle/PI*180 << " deg" << std::endl;
	SendAngle(angle);

	// Request the current position. The method OnReceivedReadPosition is called when the MIP device sends the position.
	SendReadPosition();

	// Request the axix status. The method OnReceivedReadAxisStatus is called when the axis status is received.
	SendReadAxisStatus();

	// This schedules this method to be called again after 1 second.
	SetNextEvent(1000);
	return;
}

void THISCLASS::OnReceivedReadPosition(int cmdid, int position) {
	// The MIP sends us the current motor position (because we requested that with SendReadPosition).
	double angle=PositionToAngle(position);
	std::cout << "Current position: " << angle/PI*180 << " deg" << std::endl;
}

void THISCLASS::OnReceivedReadAxisStatus(int cmdid, MipDevice::eAxisStatusFlags status) {
	// The MIP sends us its current axis status (because we requested that with SendReadAxisStatus).

	// Flags
	mMoving=(status & sAxisStatusMoving == sAxisStatusMoving);
	mHome=(status & sAxisStatusHome == sAxisStatusHome);
	mReferenced=(status & sAxisStatusReferenced == sAxisStatusReferenced);

	// Error available
	if (status & sAxisStatusError) {
		SendReadError(0);
	}

	// Warning available
	if (status & sAxisStatusWarning) {
		SendReadWarning(0);
	}

	// If the absolute position is not known, find it now
	if (! mReferenced) {
		SendSetPosVelocity(0, 10);
		SendFindHomeSys(0);
	}
}

void THISCLASS::OnReceivedReadError(int cmdid, MipDevice::eError error) {
	// The MIP sends us the error code (because we requested that with SendReadError).
	mError=error;
	if (mError==0) {return;}

	if ((mError>0) && (mError<=10)) {
		// If the error code is in 1..10, there is a fatal hardware error. Stop the motion.
		mDriveMotor=false;
		SendStopMotion();
		std::cout << "MIP Fatal Error " << mError << ": stopping motion, check the hardware" << std::endl;
	} else {
		// Otherwise, just try to clear the error. Depending on the application, it might
		// not be good to override these errors.
		SendClearError(0);
		std::cout << "MIP Error " << mError << ": trying to clear the error" << std::endl;
	}
}

void THISCLASS::OnReceivedReadWarning(int cmdid, MipDevice::eWarning warning) {
	// The MIP sends us the warning code (because we requested that with SendReadWarning).
	mWarning=warning;
	if (mWarning==0) {return;}

	// Try to clear the warning. Depending on the application, it might not be good to
	// override the warnings. In this simple example, we don't care about warnings.
	SendClearWarning(0);
	std::cout << "MIP Warning " << mWarning << ": trying to clear the warning" << std::endl;
}

int THISCLASS::AngleToPosition(double angle) {
	return (int)(angle/PI*180*5.5556*531);
}

double THISCLASS::PositionToAngle(int position) {
	return (double)position/5.5556/531/180*PI;
}
